
function [FD]=Main_AnalysisLL(file_name,file_param,file_stime,file_tau,vi,lambda,type_fun,type_method,est_par_ig,gammino,...
    soglia,x0,x1,type_sigma,type_pizero,type_estdegree,type_test,type_mod,nfirst,...
    L,estpizeroC,estsigmaC,file_matrix,iplot,file_ps,overwrite,scale,t0)
   clear lasterr
close(Analysis);
hhh=waitbar(0,'Please wait...');

% This program perform the real data application presented in the article "Bayesian approach to estimation and 
% testing in time course microarray experiments" by C. Angelini, D. De
% Canditiis, M. Mutarelli and M. Pensky (2007)
 
%
% Data are normalized and adjusted in a matrix named "datamatrix" where each column is
% a microarray and each line represents the expression profile of a gene. Aim of the analysis is to detect 
% which function is significantly different from zero and for those functions to estimate the underlying profile
% 
% 
% Reference: 
% C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The user have to provide the following input parameters

% file_name:     name of the file where to write the output results

% vi_global:     real number in [0,1], represent the expected number of derivatives for s(t). It is a global parameter, default value 0;  
% lambda:        real positive number.  Poisson parameter for the prior probability on the polynomial degree
%                Note, that the Poisson is truncated, hence lambda does not represent the average expected degree 
% type_fun :     string that can take values 'legendre' or 'fourier' 
% L:             integer number, that is the maximum degree of polynomials representing the expressed profile
% type_method:   integer number 1,2,3 represents the selected model; 1 delta distribution on sigma^2; 2 IG distribution
%                on sigma^2;3 is a Exponential distribution on sigma^2
% est_par_ig:    integer number 1,2,3; it is necessary only in the case of method 2 and it is the option on how to 
%                estimate parameter [b gammino] of the distribution IG
%                '1' --> gammino is given and b is evaluated s.t. the prior mean of sigma is the estimated sigma
%                '2' --> gammino is obtained as the parameter estimate for gamma distributed data applied over
%                      the inverse of the sampled variances of the microarrays
%                '3' --> gammino and b are estimated simultaneously by  MLE using all the data under  'soglia'
% gammino:       positive real number, to be provided only if est_par_ig==1
% soglia:        positive real number, to be provided only if est_par_ig==3
% x0,x1:         positive real numbers, are the interval where the minimization of the marginal pdf is done to estimate tau2
 
% type_sigma:     string that can take values 'standard' or  'mad', it is the option on how to estimate sigma.
%                'standard'--> standard deviation of the noise
%                'mad'--> median absolute deviation of the noise.
% type_pizero:    string is the option on how to estimate pizero. 
%                'universal' --> (with an Hard Universal Threshold)
%                'sure' --> (Adaptive Threshold Selection Using Principle of SURE)
%                'EBL'; --> (Empirical Bayes threshold)
% type_estdegree: string that is the option on how to estimate the degree. 
%                'mean' --> (posterior mean estimation )
%                'map' --> (Maximum posterior estimation)
% type_test:      string that is the option on which test procedure is used.
%                 'binomial'
%                 'standard_odds'
%                 'standard_BF'
%                 'Pois2';
%                 'Full';
% type_mod:       string that is the option on multiple testing hypothesis procedure
%                 'global' 
%                 'Step-Up';
%                 'Step-Down';
% iplot:           integer indicator used to decide if to plot the selected profiles
%                 if iplot=0 then we plot the estimated profiles, otherwise the plots are not produced.
% nfirst:          Integer,  Choose how many gene profile you want to plot (since they are ranked the first profiles are the most significant!!)
 
% For standard usage the users have to provide the following input
% parameters : (here the parameters are stored in the data-structure avellino)
%-----------------------------------------------------------------------------------------------
% 
% times:         vector of positive real number; the time grid knots 
% repliche:      vector of positive integer number (with the same size of time) each component is
%                the number of replication at the correspondent time point 
% datamatrix:    Matrix of your data Y(t). it should be stored according to the structure given in vectors times and repliche 
%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%        Load data                                                  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Load dataset ......\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isempty(file_name)
    aaa=isdir(fullfile('UserData'));
    if aaa==0
        mkdir('UserData');
    end
    file_name=fulfile('UserData','simulation.out');
end

if strcmp(file_matrix(end-3:end),'.xls')

    [data,names]=xlsread(file_matrix);
    times_vet=data(1,:);
    times=unique(times_vet);
    repliche=zeros(size(times));
    for i=1:length(times)
    repliche(i)=length(find(times_vet==times(i)));
    end
    gene_names=names(2:end,1);
    datamatrix=data(2:end,:);
    n=length(gene_names);
    clear data names
elseif strcmp(file_matrix(end-3:end),'.txt')
    A=importdata(file_matrix);
    times_vet=A.data(1,:);
    times=unique(times_vet);
    repliche=zeros(size(times));
    for i=1:length(times)
    repliche(i)=length(find(times_vet==times(i)));
    end
    gene_names=A.textdata(2:end,1);
    datamatrix=A.data(2:end,:);
    n=length(gene_names);
    clear A
else
fprinf('File format not supported....\n ');
return
end
if length(times)<=5
    fprintf('warning: BATS is designed for time series experiments having\n more then 5 time points!')
    fprintf('The analysis will be performed, but we suggest to use a different tool.\n')
end
if isempty(L)
    L=ceil(length(times)/2);
end
if isempty(lambda)
    lambda=expected_degree(L);
end

fprintf('File %s ......\n', file_matrix); 
fprintf('Done ......\n');
t1=cputime;
dt=t1-t0;
fprintf('******Data and parameters were loaded in %3.8f seconds******\n',dt); 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    SET-UP input parameter  for the routine Make_Analysis          
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
% type_fun='legendre';  %Select your favorite basis : type_fun='legendre'; or type_fun='fourier';
% if strcmp(type_fun,'legendre')
%  L=6;  %expected maximum degree of polynomial
%  vi=0;  %expect s(t) vi-times differentiable
%  lambda=11;  % Poisson parameter for the prior probability on the polynomial degree
%  x0=10^(-8);   x1=70; %%% Extremes of the interval for maximization of the marginal pdf to find tau2
%  
% elseif strcmp(type_fun,'fourier') 
%  L=6;  %expected maximum degree of polynomial
%  vi=1.;  %expect s(t) vi-times differentiable
%  lambda=3;  %Poisson parameter for the prior probability on the polynomial degree
%  x0=10^(-8); x1=30^2;  %%% Extremes of the interval for maximization of the marginal pdf to find tau
% end
%  
% type_method=3;%1 delta distribution on sigma^2; 2 IG distribution on sigma^2;3 is a Exponential dist on sigma^2
% soglia=[];  % this is the threshold for the l_2 norm of the included profile for the estimation of parameters b 
%             % and gammino in the case of type_method==2 and est_par_ig==3
% if type_method==2
%   if exist('est_par_ig','var')==0, est_par_ig=2; end  
%   if est_par_ig==1
%   if exist('gammino','var') == 0,  gammino=15; end 
%   else
%       gammino=[];
%   end
%   if est_par_ig==3
%    if ~isempty(soglia),  soglia=1.5; end
%   end
% else
%     est_par_ig=[];
%     gammino=[];
% end
%  
% type_sigma='standard'; %option on how to estimate the standard deviation of the noise.
% type_pizero='universal'; %option on how to estimate pizero.  
% type_estdegree='map'; %option on how to estimate the degree.
% type_test='binomial'; % option on which test procedure is used.
% if strcmp(type_test,'standard_odds') || strcmp(type_test,'standard_BF') || strcmp(type_test,'binomial') 
%      type_mod=[];
% elseif strcmp(type_test,'Pois2') 
%     type_mod='global';  % type_mod='Step-Up'; type_mod='Step-Down';
% end
%%%%%%%%%%%%%%%%%%%%%%% END SETUP PARAMETER%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Find the "significant" genes     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nfig=0;
% iplot=0; % if iplot=0 then plots the estimated profiles, otherwise the plot is not produced.
% nfirst=1500; %Chose How many gene profile you want plot (since they are ranked the first profile are the most significant!!)
nt=size(datamatrix,2);

vectnan=sum(isnan(datamatrix),2);


i50pernan=find(vectnan>=nt/3);
FD=0;
if ~isempty(i50pernan)
    fprintf('Too many missing values for some records.\n Please filter out your data before repeating the analysis.\n You can use the "Filter Data" BATS utility tool.\n' )
    h=msgbox({'Too many missing values for some records.';
         'Please filter out your data before repeating the analysis.';
         'You can use the "Filter Data" BATS utility tool.'},'Please filter out the data!','Error');
     FD=1;
     uiwait(h)
     close all

    return
end
fprintf('Number of missing spots in the dataset: %i on a total of %i spots\n', sum(vectnan),size(datamatrix,1)*size(datamatrix,2));
t2=cputime;
waitbar(1/3)
% [id_rejected,BF,esttau2,Lhat,estpizero,estsigma]=Make_Analysis(datamatrix,times,repliche,L,vi,lambda,type_fun,type_method,type_sigma,type_pizero,...
%     type_estdegree,type_test,type_mod,est_par_ig,gammino,soglia,x0,x1);
[id_rejected,BF,esttau2,Lhat,estpizero,estsigma]=Make_Analysis(datamatrix,times,repliche,L,vi,lambda,type_fun,...
    type_method,type_sigma,type_pizero,type_estdegree,type_test,type_mod,est_par_ig,gammino,soglia,x0,x1,estsigmaC,estpizeroC);
t3=cputime;
dt=t3-t2;
fprintf('******Gene identified in %3.5f seconds******\n',dt);
waitbar(2/3)

flagtau2=0;
if max(esttau2)> x1-5
     if iplot==0   
      if overwrite==1 
          idtau=fopen(file_tau);
          if idtau~=-1
              fclose(idtau);
              delete(file_tau)
          end
      end
    
      nfig=nfig+1;
      h=figure(nfig)
      plot(1:n,esttau2)
      title('Estimated parameter \tau^2 ')
      
      ss=sprintf('print -f%d -dpsc %s -append',h,file_tau)
      eval(ss)
      close(h)
      
     end
    taupper=find(esttau2>x1-5);
    fprintf('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n');
    fprintf('Suggest to repeat the analysis increasing the value of x1 \n');
    fprintf('Number of genes exceeding the upper bound for tau %i \n',length(taupper));
    fprintf('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n');
    flagtau2=1;
end
 
if ~isempty(id_rejected)
  [aa,ii]=sort(BF(id_rejected));
  sort_id_rejected=id_rejected(ii);
  nrejected=length(id_rejected);
else
  nrejected=0;
end
 
prop=nrejected/n;
fprintf('Summary results of the analysis......\n');
if strcmp(type_test,'Full')
    fprintf('Full list of genes ranked according to the BF is provided!\n');
else
    fprintf('Number of genes detected as significant %i which are %3.3f of the total......\n',nrejected,prop);
end
t4=cputime;
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimate significant profiles
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

waitbar(2.5/3)
if strcmp(type_fun,'legendre')
     if nrejected>0
       [coeffs_hat,shat]=Make_Estimation(datamatrix,sort_id_rejected,BF(sort_id_rejected),Lhat(sort_id_rejected),estpizero,esttau2(sort_id_rejected),L,vi,type_fun,times,repliche);
     end
     t5=cputime;
     dt=t5-t4;
     fprintf('******Profiles Estimated in %3.5f seconds******\n',dt);
  if ispc   
     %writing shat profiles
     header={'Rank'; 'Gene Names'};
     xlswrite(file_stime,header',1,'A1:B1');
     letters={'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z',...
        'AA','AB','AC','AD','AE','AF','AG','AH','AI','AJ','AK','AL','AM','AN','AO','AP','AQ','AR','AS','AT','AU','AV','AW','AX','AY','AZ',...
        'BA','BB','BC','BD','BE','BF','BG','BH','BI','BJ','BK','BL','BM','BN','BO','BP','BQ','BR','BS','BT','BU','BV','BW','BX','BY','BZ',...
        'CA','CB','CC','CD','CE','CF','CG','CH','CI','CJ','CK','CL','CM','CN','CO','CP','CQ','CR','CS','CT','CU','CV','CW','CX','CY','CZ',...
        'DA','DB','DC','DD','DE','DF','DG','DH','DI','DJ','DK','DL','DM','DN','DO','DP','DQ','DR','DS','DT','DU','DV','DW','DX','DY','DZ'};
    nt=length(times);

%save coeffs_hat coeffs_hat
   % for gg=1:nt
%         times1(gg)={sprintf('Shat(%2.2d)',times(gg))};
%    end

    rangeT=sprintf('C1:%s1',letters{nt+2});
    
%     xlswrite(file_stime,times1,rangeT);
 xlswrite(file_stime,times,rangeT);
 RangeR=sprintf('A2:A%d',nrejected+1);
 RangeG=sprintf('B2:B%d',nrejected+1);
 RangeSH=sprintf('C2:%s%d',letters{nt+2},nrejected+1);
 
 
 xlswrite(file_stime,(1:nrejected)',1,RangeR);
 xlswrite(file_stime,gene_names(sort_id_rejected),1,RangeG);
 xlswrite(file_stime,shat,1,RangeSH);
else

 fprintf('Estimated profiles are saved in .txt format\n');
 fidtxt = fopen(file_stime,'wt'); 
     fprintf(fidtxt,'Rank\t Gene_Names\t');
    for itimes=1:length(times)
     fprintf(fidtxt,'%f \t',times(itimes)); 
    end
     fprintf(fidtxt,'\n');
    for irej=1:nrejected
     fprintf(fidtxt,'%i \t %s \t',irej,gene_names{sort_id_rejected(irej)});
     for itimes=1:length(times)
     fprintf(fidtxt,'%f \t',shat(irej,itimes));
     end
      fprintf(fidtxt,'\n');
    end
    fclose(fidtxt);
end

   
 if iplot==0   
     minshat=min(min(shat));
     maxshat=max(max(shat));
      if overwrite==1 
           idps=fopen(file_ps);
          if idps~=-1
              fclose(idps);
              delete(file_ps)
          end
      end
       counter=0;
 nfig=nfig+1;
% figstart=nfig;
 for i=1:min(nfirst,nrejected)
     counter=counter+1;    
    h=figure(nfig);
    subplot(5,3,counter)
    h=plot_profile(sort_id_rejected(i),datamatrix,times,repliche,nfig);
    hold on
    plot(times,shat(i,:),'g')
    if scale==1
          %case globalscale
          axis([min(times) max(times) min([minshat datamatrix(sort_id_rejected(i),:)]) max([maxshat datamatrix(sort_id_rejected(i),:)])])
    else
          %case autoscale
          axis([min(times) max(times) min([shat(i,:) datamatrix(sort_id_rejected(i),:)]) max([shat(i,:) datamatrix(sort_id_rejected(i),:)])])
    end


    hold off
    tit=['Estimation of ' gene_names{sort_id_rejected(i)}];
    title(tit)
    ifig=mod(i,15);
    if ifig==0
         counter=0;
         nfig=nfig+1;
%          if i==1
%              ss=sprintf('print -f%d -dpsc %s',h,file_ps);
%          else
%              ss=sprintf('print -f%d -dpsc %s -append',h,file_ps);
%          end
%             

            ss=sprintf('print -f%d -dpsc %s -append',h,file_ps);
            %              ss=sprintf('print -f%d -dpdf %s -append',h,file_ps);

                          
         eval(ss)
         close(h)
     end
      
 end 
 if ifig~=0
     figure(h)
%  if i==1
%      ss=sprintf('print -f%d -dpsc %s',h,file_ps);
%  else
%      ss=sprintf('print -f%d -dpsc %s -append',h,file_ps);
%  end
     ss=sprintf('print -f%d -dpsc %s -append',h,file_ps);
     eval(ss)
     close(h)
 end
%  figend=figure;
%  for ifig=figstart:figend
%       figure(ifig)
% %      set(gcf,'PaperType','A4'); 
% %      set(gcf,'paperpositionmode','manual'); 
% %      set(gcf,'paperposition',[0 0 8 12]); 
% %      set(gcf,'PaperUnits', 'centimeters'); 
% %  
%       print -dpsc profili_legendre.ps -append
%  end

 end
elseif strcmp(type_fun,'fourier')
 
if nrejected>0
 [coeffs_hat,shat]=Make_Estimation(datamatrix,sort_id_rejected,BF(sort_id_rejected),Lhat(sort_id_rejected),estpizero,esttau2(sort_id_rejected),L,vi,type_fun,times,repliche);
end
t5=cputime;
dt=t5-t4;
fprintf('Profiles Estimated in %3.5f seconds\n',dt);
%save coeffs_hat coeffs_hat
 %writing shat profiles
 if ispc
     header={'Rank'; 'Gene Names'};
     xlswrite(file_stime,header',1,'A1:B1');
     letters={'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z',...
        'AA','AB','AC','AD','AE','AF','AG','AH','AI','AJ','AK','AL','AM','AN','AO','AP','AQ','AR','AS','AT','AU','AV','AW','AX','AY','AZ',...
        'BA','BB','BC','BD','BE','BF','BG','BH','BI','BJ','BK','BL','BM','BN','BO','BP','BQ','BR','BS','BT','BU','BV','BW','BX','BY','BZ',...
        'CA','CB','CC','CD','CE','CF','CG','CH','CI','CJ','CK','CL','CM','CN','CO','CP','CQ','CR','CS','CT','CU','CV','CW','CX','CY','CZ',...
        'DA','DB','DC','DD','DE','DF','DG','DH','DI','DJ','DK','DL','DM','DN','DO','DP','DQ','DR','DS','DT','DU','DV','DW','DX','DY','DZ'};
    nt=length(times);


%     for gg=1:nt
%         times1(gg)={sprintf('Shat(%2.2d)',times(gg))};
%     end

    rangeT=sprintf('C1:%s1',letters{nt+2});
    

 xlswrite(file_stime,times,rangeT);
 RangeR=sprintf('A2:A%d',nrejected+1);
 RangeG=sprintf('B2:B%d',nrejected+1);
 RangeSH=sprintf('C2:%s%d',letters{nt+2},nrejected+1);
 
 
 xlswrite(file_stime,(1:nrejected)',1,RangeR);
 xlswrite(file_stime,gene_names(sort_id_rejected),1,RangeG);
 xlswrite(file_stime,shat(:,nt+1:2*nt),1,RangeSH);
 else
 
 
 fprintf('Estimated profiles are saved in .txt format\n');
 fidtxt = fopen(file_stime,'wt'); 
     fprintf(fidtxt,'Rank\t Gene_Names\t')
    for itimes=1:length(times)
     fprintf(fidtxt,'%f \t',times(itimes)); 
    end
     fprintf(fidtxt,'\n');
    for irej=1:nrejected
     fprintf(fidtxt,'%i \t %s \t',irej,gene_names{sort_id_rejected(irej)});
     for itimes=1:length(times)
     fprintf(fidtxt,'%f \t',shat(irej,itimes));
     end
      fprintf(fidtxt,'\n');
    end
    fclose(fidtxt);
 end
   
 if iplot==0   
      if overwrite==1 
          idps=fopen(file_ps);
          if  idps~=-1
              fclose(idps);
              delete(file_ps)
          end
      end
  minshat=min(min(shat(:,nt+1:2*nt)));
  maxshat=max(max(shat(:,nt+1:2*nt)));
 counter=0;
 nfig=nfig+1;
% figstart=nfig;
 for i=1:min(nfirst,nrejected)
     counter=counter+1;    
    h=figure(nfig);
    subplot(5,3,counter)
    h=plot_profile(sort_id_rejected(i),datamatrix,times,repliche,h);
    hold on
    
    plot(times,shat(i,nt+1:2*nt),'g')  
    %plot(times,shat(i,:),'g')  
    if scale==1
        %case globalscale
        axis([min(times) max(times) min([minshat datamatrix(sort_id_rejected(i),:)]) max([maxshat datamatrix(sort_id_rejected(i),:)])])
    else
        %case autoscale
        axis([min(times) max(times) min([shat(i,nt+1:2*nt) datamatrix(sort_id_rejected(i),:)]) max([shat(i,nt+1:2*nt) datamatrix(sort_id_rejected(i),:)])])
    end
    

    %axis([min(times) max(times) min([shat(i,:) datamatrix(sort_id_rejected(i),:)]) max([shat(i,:) datamatrix(sort_id_rejected(i),:)])])
    hold off
    tit=['Estimation of ' gene_names{sort_id_rejected(i)}];
    title(tit)
     ifig=mod(i,15);
     if ifig==0
         counter=0;
         nfig=nfig+1;  
%          if i==1
%              ss=sprintf('print -f%d -dpsc %s',h,file_ps);
%          else
%              ss=sprintf('print -f%d -dpsc %s -append',h,file_ps);
%          end
         ss=sprintf('print -f%d -dpsc %s -append',h,file_ps);
         eval(ss)
         close(h)
     end
   
 end 
  if ifig~=0
         figure(h) 
%          if i==1
%              ss=sprintf('print -f%d -dpsc %s',h,file_ps);
%          else
%              ss=sprintf('print -f%d -dpsc %s -append',h,file_ps);
%          end
         ss=sprintf('print -f%d -dpsc %s -append',h,file_ps);
         eval(ss)
         close(h)
  end
end
end
waitbar(1)

fid = fopen(file_param,'wt'); 
fprintf(fid,'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n');
fprintf(fid,' SUMMARY REPORT ANALYSIS...\n');
fprintf(fid,'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n');
fprintf(fid,'\n');
fprintf(fid, 'Analizing file %s ......\n', file_matrix); 
fprintf(fid,'\n');
fprintf(fid, 'Number of genes %i in the study, number of arrays %i, \n',size(datamatrix,1),size(datamatrix,2));
fprintf(fid,'\n');
fprintf(fid,'Arrays taken at times [ '); 
fprintf(fid,'%3.2f ',times_vet);
fprintf(fid,'] \n');
fprintf(fid,'\n');
fprintf(fid,' Number of missing spots in the dataset: %i on a total of %i spots\n', sum(vectnan),size(datamatrix,1)*size(datamatrix,2));
maxmiss=max(vectnan);
fprintf(fid,'Number of genes without missing values %i, \n',sum(vectnan==0)); 
for imiss=1:maxmiss
 fprintf(fid,'Number of genes with %i missing values %i,\n',imiss,sum(vectnan==imiss));   
end
fprintf(fid,' No genes with more than %i missings\n',maxmiss);
   
fprintf(fid,'------------------------------ \n');

fprintf(fid,' Basis : %s \n',type_fun);
fprintf(fid,' Expected maximum degree of polynomial : %i \n',L);
fprintf(fid,' S(t) expected to be vi-times differentiable : %i \n',vi);
fprintf(fid,' Poisson parameter for the prior probability g(L_i) : %i \n ',lambda);
fprintf(fid,' \n' );
fprintf(fid,' ESTIMATION METHOD = %i    \n',type_method );
 
if type_method ==2
 fprintf(fid,' Option on IG-hyper-parameters estimations %i \n',est_par_ig);
 if est_par_ig==1
 fprintf(fid,' User chosen parameter "gamma"  %i \n',gammino);
 end
end
 
fprintf(fid,' Interval where searching tau2 is [ %1.5f , %1.5f ] \n',x0,x1);
fprintf(fid,' How to estimate sigma= %s \n',type_sigma);
fprintf(fid,' How to estimate pizero= %s \n',type_pizero);
fprintf(fid,' How to estimate Lhat for each function= %s \n',type_estdegree);
fprintf(fid,' How to control multiplicity= %s \n',type_test);
if strcmp(type_test,'Full')
    fprintf(fid,'NOTE: FULL LIST of genes ranked according to the BF is provided!\n');
end




fprintf(fid,' \n' ); 
fprintf(fid,'------------------------------ \n');
fprintf(fid,' Number of genes in the study  %i \n',n);
fprintf(fid,' Number of  detected as "differentially expressed" genes %i \n',nrejected);
fprintf(fid,' Estimation sigma  %1.5f \n',estsigma);
fprintf(fid,'Estimation pizero  %1.5f \n',estpizero);
fprintf(fid,'                 \n');
fprintf(fid,'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% \n');
 
if flagtau2==1
  fprintf(fid,'xxxxxxxxxxxxxxxxxxxxxxxxxx\n');
  fprintf(fid, 'Suggest to repeat the analysis increasing the value of x1 \n');
  fprintf(fid,'Number of genes exceeding the upper bound for tau %i, namely \n',length(taupper));
  for idmiss=1:length(taupper)
      fprintf(fid,'Gene %s \n',gene_names{taupper(idmiss)});
  end
  fprintf(fid,'xxxxxxxxxxxxxxxxxxxxxxxxxx\n');
end

fclose(fid);

%fid = fopen(file_name,'wt');

%fprintf(fid,'%%%%%%%%%%%% List of selected genes %%%%%%%%%% \n');
 
if nrejected>0
    if ispc
header={'Rank'; 'Gene Name'; 'Estimated Lhat'; 'BF'; 'L2-norm'; 'Nmiss'};
xlswrite(file_name,header',1,'A1:F1');
 RangeR=sprintf('A2:A%d',nrejected+1);
 RangeG=sprintf('B2:B%d',nrejected+1);
 RangeLH=sprintf('C2:C%d',nrejected+1);
 RangeBF=sprintf('D2:D%d',nrejected+1);
 RangeL2=sprintf('E2:E%d',nrejected+1);
 RangeNm=sprintf('F2:F%d',nrejected+1);
 
 xlswrite(file_name,(1:nrejected)',1,RangeR);
 xlswrite(file_name,gene_names(sort_id_rejected),1,RangeG);
 xlswrite(file_name,Lhat(sort_id_rejected)',1,RangeLH);
 xlswrite(file_name,BF(sort_id_rejected)',1,RangeBF);
% 
l2=zeros(size(nrejected,1));
nmiss=zeros(size(nrejected,1));
 for i=1:nrejected
  l2(i,1)=sqrt(max(times)/2)*sum(coeffs_hat{i}.^2);
  nmiss(i,1)=sum(isnan(datamatrix(sort_id_rejected(i),:)));
 end
 
xlswrite(file_name,l2,1,RangeL2);
xlswrite(file_name,nmiss,1,RangeNm);

    else
        
 l2=zeros(size(nrejected,1));
nmiss=zeros(size(nrejected,1));
 for i=1:nrejected
  l2(i,1)=sqrt(max(times)/2)*sum(coeffs_hat{i}.^2);
  nmiss(i,1)=sum(isnan(datamatrix(sort_id_rejected(i),:)));
 end       
        
 fprintf('List of selected genes are saved in .txt format\n');
 fidtxt = fopen(file_name,'wt'); 
     fprintf(fidtxt,'Rank\t Gene_Name\t Estimated_Lhat\t BF\t L2-norm\t Nmiss\n');
   
     for irej=1:nrejected
      fprintf(fidtxt,'%i\t %s\t %i\t %e\t %e\t %i\n',irej,gene_names{sort_id_rejected(irej)},Lhat(sort_id_rejected(irej)),BF(sort_id_rejected(irej)),l2(irej,1),nmiss(irej));
     end
    fclose(fidtxt);
clear l2 nmiss

end
end

close(hhh)
