function []=Main_simulationL(file_name,vi_global,lambda,type_fun,type_method,type_error,df,est_par_ig,gammino,...
    soglia,x0,x1,type_sigma,type_pizero,type_estdegree,type_test,type_mod,n,ndiff,truesigma,times,repliche,...
    L,Lmax,nn_function,nn_run,randnstate,a,b,NaNT,estpizeroC,estsigmaC,savedata,file_matrix)
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code performs a simulation of the method proposed in the manuscript 
% "Bayesian approach to estimation and  testing in time course microarray experiments" 
% by C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky.
 
%  Synthetic dataset are created according to the model datamatrix= D c + noise
%
%  The analysis is then applied to the synthetic dataset
%  The result are collected and printed in a summary report in 'file_name'   
 
% The user may provide the following input parameters for the analysis of the synthetic dataset:
%-----------------------------------------------------------------------------------------------
% file_name:     name of the file where to write the summary result
% vi_global:     real number in [0,1], represent the expected number of derivatives for s(t). It is a global parameter, default value 0;  
% lambda:        real positive number.  Poisson parameter for the prior probability on the polynomial degree
%                Note, that the Poisson is truncated, hence lambda does not represent the average expected degree 
% type_fun :     string that can take values 'legendre' or 'fourier' (NOTE 'fourier' options still under development)
% type_method:   integer number 1,2,3 represents the selected model; 1 delta distribution on sigma^2; 2 IG distribution
%                on sigma^2;3 is a Exponential distribution on sigma^2
% type_error:    string that can take values 'normal' or 'student'; this is the distribution of the noise to add to the
%                profiles
% df:            integer number; it is necessary only in the case of student noise it represents the degree of freedom
%                of the distribution 
% est_par_ig:    integer number 1,2,3; it is necessary only in the case of method 2 and it is the option on how to 
%                estimate parameter [b gammino] of the distribution IG
%                '1' --> gammino is given and b is evaluated s.t. the prior mean of sigma is the estimated sigma
%                '2' --> gammino is obtained as the parameter estimate for gamma distributed data applied over
%                      the inverse of the sampled variances of the microarrays
%                '3' --> gammino and b are estimated simultaneously by  MLE using all the data under  'soglia'
% gammino:       positive real number, to be provided only if est_par_ig==1
% soglia:        positive real number, to be provided only if est_par_ig==3
% x0,x1:         positive real numbers, are the interval where the minimization of the marginal pdf is done to estimate tau2
 
% type_sigma:     string that can take values 'standard' or  'mad', it is the option on how to estimate sigma.
%                'standard'--> standard deviation of the noise
%                'mad'--> median absolute deviation of the noise.
% type_pizero    string is the option on how to estimate pizero. 
%                'universal' --> (with an Hard Universal Threshold)
%                'sure' --> (Adaptive Threshold Selection Using Principle of SURE)
%                'EBL'; --> (Empirical Bayes threshold)
% type_estdegree: string that is the option on how to estimate the degree. 
%                'mean' --> (posterior mean estimation )
%                'map' --> (Maximum posterior estimation)
% type_test:      string that is the option on which test procedure is used.
%                 'binomial'
%                 'standard_odds'
%                 'standard_BF'
%                 'Pois2';
% type_mod:       string that is the option on multiple testing hypothesis procedure
%                 'global' 
%                 'Step-Up';
%                 'Step-Down';
 
% The user may provide the following input parameters for generating the synthetic data set:
%-----------------------------------------------------------------------------------------------
 
% n:             integer number, the number of genes we want to study
% ndiff:         integer number, the number of differentially expressed genes (of course ndiff<n)
% truesigma      positive real number, the std of the noise 
% times:         vector of positive real number; the time grid knots 
% repliche:      vector of positive integer number (with the same size of time) each component is
%                the number of replication at the correspondent time point 
% L:             integer number, that is the maximum degree of polynomials representing the expressed profile
 
 
% nn_function:   integer number; it is the number of different set of functions differentially ezpressed 
% nn_run=3:      integer number; it is the number of runs to perform for each set of functions
%                (In each run we have a different noise realization)
% randnstate:    integer number; set a different initialization for the random number generator 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear lasterr
%close(Analysis)

if isempty(file_name)
    aaa=isdir(fullfile('Data'));
    if aaa==0
        mkdir('Data');
    end
    file_name=fullfile('Data','simulation.out');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   SET-UP input parameter  for the routine Make_Analysis
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 
% A=textread(input_file);
% 
%   
% vi_global=A(1,1);    
% lambda=A(6,1);        
% dum=A(2,1);
% if dum==1
%     type_fun='legendre';
% elseif dum==2
%     type_fun='fourier';     %string that can take values 'legendre' or 'fourier' (NOTE 'fourier' options still under development)
% end
% type_method=A(4,1);   %integer number 1,2,3 represents the selected model; 1 delta distribution on sigma^2; 2 IG distribution
% %          on sigma^2;3 is a Exponential distribution on sigma^2
% dum=A(3,1);
% if dum==1
%     type_error='normal';    %string that can take values 'normal' or 'student'; this is the distribution of the noise to add to the
% %                profiles
% elseif dum==2
%     type_error='student';
%     df=A(2,2);
%             %integer number; it is necessary only in the case of student noise it represents the degree of freedom
% %                of the distribution 
% end
% if type_method==2
%     est_par_ig=A(4,2);
%     if est_par_ig==1;
%         gammino=A(4,3);
%     elseif est_par_ig==3
%         soglia=A(4,3);
% %integer number 1,2,3; it is necessary only in the case of method 2 and it is the option on how to
% %                estimate parameter [b gammino] of the distribution IG
% %                '1' --> gammino is given and b is evaluated s.t. the prior mean of sigma is the estimated sigma
% %                '2' --> gammino is obtained as the parameter estimate for gamma distributed data applied over
% %                      the inverse of the sampled variances of the microarrays
% %                '3' --> gammino and b are estimated simultaneously by  MLE using all the data under  'soglia'
% %gammino:       %positive real number, to be provided only if est_par_ig==1
% %soglia:        %positive real number, to be provided only if est_par_ig==3
% 
%     end
% end
% 
% x0=A(5,1);
% x1=A(5,2);         %positive real numbers, are the interval where the minimization of the marginal pdf is done to estimate tau2
%  
% dum=A(7,1);
% if dum==1;
%     type_sigma='standard';
% elseif dum==2
%     type_sigma='mad';
% end
%     
% %type_sigma:   %  string that can take values 'standard' or  'mad', it is the option on how to estimate sigma.
% %                'standard'--> standard deviation of the noise
% %                'mad'--> median absolute deviation of the noise.
% dum=A(8,1);
% if dum==1;
%     type_pizero='universal';
% elseif dum==2
%     type_pizero='sure';
% elseif dum==3
%     type_pizero='EBL';
% end
% 
% %type_pizero    %string is the option on how to estimate pizero. 
% %                'universal' --> (with an Hard Universal Threshold)
% %                'sure' --> (Adaptive Threshold Selection Using Principle of SURE)
% %                'EBL'; --> (Empirical Bayes threshold)
% dum=A(20,1)
% if dum==1
%     type_estdegree='map';
% elseif dum==2
%     type_estdegree='mean';
% end
% %type_estdegree: %string that is the option on how to estimate the degree. 
% %                'mean' --> (posterior mean estimation )
% %                'map' --> (Maximum posterior estimation)
% 
% dum=A(9,1);
% if dum==1
%     type_test='binomial';
% elseif dum==2
%     type_test='standard_odds';
% elseif dum==3
%     type_test='standard_BF';
% elseif dum==4
%     type_test='Pois2';
% end
%                 
% %type_test:      %string that is the option on which test procedure is used.
% %                 'binomial'
% %                 'standard_odds'
% %                 'standard_BF'
% %                 'Pois2';
% dum=A(10,1);
% if dum==1
%     type_mod='global';
% elseif dum==2
%     type_mod='Step-Up';
% elseif dum==3
%     type_mod='Step-Down';
% end
% %type_mod:       %string that is the option on multiple testing hypothesis procedure
% %                 'global' 
% %                 'Step-Up';
% %                 'Step-Down';
%  
% % The user may provide the following input parameters for generating the synthetic data set:
% %-----------------------------------------------------------------------------------------------
%  n=A(11,1);
% %n:        integer number, the number of genes we want to study
% ndiff=A(12,1);
% %ndiff:         integer number, the number of differentially expressed genes (of course ndiff<n)
% truesigma=A(13,1);
% % truesigma      positive real number, the std of the noise
% nt=A(14,1);
% times=[];
% for i=2:nt+1
%     times=[times,A(14,i)];
% end
% % times:         vector of positive real number; the time grid knots 
% nr=A(15,1);
% repliche=[];
% for i=2:nr+1
%     repliche=[repliche,A(15,i)];
% end
% % repliche:      vector of positive integer number (with the same size of time) each component is
% %                the number of replication at the correspondent time point 
% L=A(16,1);
% % L:             integer number, that is the maximum degree of polynomials representing the expressed profile
%  
%  nn_function=A(17,1);
%  % nn_function:   integer number; it is the number of different set of functions differentially ezpressed 
% nn_run=A(18,1);
%  % nn_run=3:      integer number; it is the number of runs to perform for each set of functions
% %                (In each run we have a different noise realization)
% randnstate=A(19,1);
% % randnstate:    integer number; set a different initialization for the random number generator 
% 
% 
% 


if exist('vi_global','var')==0, vi_global=0; end  % we expect s(t) vi-times differentiable
if exist('lambda','var') == 0, lambda=9;  end    % Poisson parameter for the prior probability on the polynomial degree
if exist('type_fun','var') == 0, type_fun='legendre'; end % Select your favorite basis : type_fun='legendre'; or type_fun='fourier';
if exist('type_method','var') == 0, type_method=2; end    %1 delta distribution on sigma^2; 2 IG distribution on sigma^2;3 is a Exponential dist on sigma^2
if exist('type_error','var') == 0, type_error='student'; end % other option is type_error='student';

 
if strcmp(type_error,'student')
if exist('df','var') == 0, df=5; end  %%% choose your favorite df --> smaller is df heavier are the tails, note df>3 
if df<3
    fprintf('df should be greater or equal 3, It have been replaced with 3');
    df=3;
end
end
% est_par_ig=3;
% if type_method==2
%   if exist('est_par_ig','var') == 0, est_par_ig=2; end  % option on how to estimate parameter of IG: 1, 2 ,3
%   if est_par_ig==1
%   if exist('gammino','var') == 0, gammino=10; end 
%   else
%       gammino=[];
%   end
%   if est_par_ig==3
%    if exist('soglia','var'), soglia=1.5; end
%   end
% else
%     est_par_ig=[];
%     gammino=[];
% end
%  
%x0=10^(-8); x1=80;  % interval where looking for tau2
 
% type_sigma='standard'; %option on how to estimate the standard deviation of the noise.   
% type_pizero='universal'; %option on how to estimate pizero.  
% type_estdegree='map'; %option on how to estimate the degree.
% type_test='binomial'; % option on which test procedure is used.


if strcmp(type_test,'standard_odds') || strcmp(type_test,'standard_BF') || strcmp(type_test,'binomial') 
     type_mod=[];
elseif strcmp(type_test,'Pois2') 
    type_mod='global'; 
end
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   SET-UP  parameters to create synthetic data set
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% n=8000;  %number of genes we want to study
% ndiff=600;  %number of differentially expressed genes (of course ndiff<n)
% truesigma=0.27;  %std of the noise 
% times=[1 2 4 6 8 12 16 20 24 28 32];  % times knots 
% repliche=[2 3 2 2 3 2 3 2 2 2 2];  %number of replication for each time instant
% L=6;  % maximum degree of polynomial
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %   SET-UP  parameters for the simulation
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  
% nn_function=1; %Number of different Datasets 
% nn_run=1; %Number of runs to perform for each set of function--> In each run we have a different noise realization
% randnstate=0;
%  
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  
randn('state',randnstate);
rand('state',randnstate);
 
T=length(times);  %number of distinct times points
design_matrix=make_design_matrix(L,times,type_fun);  % construction of design matrix over the grid knots without replications 
m=sum(repliche); % m is the number of observations
 
fun_nrejected=zeros(1,nn_function);
fun_n_correctly_rejected=zeros(1,nn_function);
fun_n_not_detected=zeros(1,nn_function);
fun_n_falsely_rejected=zeros(1,nn_function);
fun_n_missclassified=zeros(1,nn_function);
 
fun_meantrueerror_a=zeros(1,nn_function);
fun_maxtrueerror_a=zeros(1,nn_function);
fun_meantrueerror_r=zeros(1,nn_function);
fun_maxtrueerror_r=zeros(1,nn_function);
fun_meanfalselyerror=zeros(1,nn_function);
fun_maxfalselyerror=zeros(1,nn_function);
fun_meanfalselynegerror=zeros(1,nn_function);
fun_maxfalselynegerror=zeros(1,nn_function);
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Simulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
flagtau2=zeros(nn_function,nn_run);
 
h = waitbar(0,'Please wait...');

for index_fun=1:nn_function
    waitbar(index_fun-(index_fun-0.5)/nn_function)
    fprintf('*****************************************   \n')
    fprintf('Current dataset %i \n',index_fun);
    fprintf('*****************************************   \n')
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Create synthetic data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %SNR b-a==4; 2==a---->range SNR nell'interfaccia; default:[2,6]
  SNR=sqrt(rand(1,ndiff)*(b-a)+a); %range of SNR 
  
  truetau=SNR*truesigma;
  truedegree=unidrnd(L,1,ndiff); %%% polynomial degree have to be generated in [1,L] , degree 0 is not allowed in this set-up
  datamatrix0=zeros(n,m);
  rdum = randperm(n); %positions of the differentially expressed genes. 
  r=sort(rdum(1:ndiff));
  v=(1:1:L+1);
  vi_gene=rand(1,ndiff);  
 
  coeff=cell(1,ndiff);
  for k=1:ndiff
     rescale=(v.^vi_gene(k));  % for each gene a different rescaling of the coefficient
     %coefficient=[];
     coefficient=truetau(k)*randn(1,truedegree(k)+1);
     coefficient=coefficient./rescale(1:truedegree(k)+1);
     coeff{k}=coefficient;
     s(k,:)=coefficient* design_matrix(1:truedegree(k)+1,:); %true expression profile over the grid time
     clear rescale; 
     clear coefficient;
   end
 %
   ss=[];%
   for k=1:T
     for l=1:repliche(k)
         ss=[ss s(:,k)]; 
     end
   end 
% 
 for k=1:ndiff
     datamatrix0(r(k),:)=ss(k,:);
 end
 clear ss;
%  
nrejected=zeros(1,nn_run);
n_correctly_rejected=zeros(1,nn_run);
n_not_detected=zeros(1,nn_run);
n_falsely_rejected=zeros(1,nn_run);
n_missclassified=zeros(1,nn_run);
 
meantrueerror_a=zeros(1,nn_run);
maxtrueerror_a=zeros(1,nn_run);
meantrueerror_r=zeros(1,nn_run);
maxtrueerror_r=zeros(1,nn_run);
meanfalselyerror=zeros(1,nn_run);
maxfalselyerror=zeros(1,nn_run);
meanfalselynegerror=zeros(1,nn_run);
maxfalselynegerror=zeros(1,nn_run);
 
for  index_run=1:nn_run
   fprintf('current run %i \n',index_run);
 
 if strcmp(type_error,'student')
   sv=sqrt(df./(df-2));
   noise=random('t',df,n,m)/sv; % if T-noise===> heavier tails noise, to see how well we perform when the assumption are violated.
                                % note Student realization is rescaled in order to produce noise with a given standard deviation 
 elseif strcmp(type_error,'normal')
  noise=randn(n,m); %% if N(0,1) , to see how we perform when the assumptions are satisfied
 else
  fprintf('Not specified type error......\n')
 end
  
datamatrix=datamatrix0+truesigma*noise; %%%%Noisy simulated data 
indNaN=find(abs(datamatrix)>NaNT);
datamatrix(indNaN)=NaN; %%% Unrealistic ("too large values") observations are filtered out
[d1,d2]=size(datamatrix);
propNaN=length(indNaN)/(d1*d2);
if propNaN>=11/100
    msg1={'The unrealistic value threshold chosen is too small! This caused too many missing data in the symulation data set. Please Choose a different threshold and rerun the program.'}
    msgbox(msg1,'Too many missing data!','Error');
    return
end
    
 
clear noise

if savedata==1
    
    %fm=length(file_matrix);
    if ispc
        if isempty(file_matrix)
            file_matrix='data.xls';
        end
     file_matrixact=sprintf('%s_%d_%d.xls',file_matrix,index_fun,index_run);
    else
     fprintf('Synthetic data are generated in a .txt format \n');
     fprintf('Data at _D_ file ; Flags are _T_ file\n');
     file_matrixact1=sprintf('%s_D_%d_%d.txt',file_matrix,index_fun,index_run);
     file_matrixact2=sprintf('%s_T_%d_%d.txt',file_matrix,index_fun,index_run);
    end


    ntimes=sum(repliche);
    times_vet=zeros(1,ntimes);
    count=0;
    for i=1:length(repliche)
        for j=1:repliche(i)
            count=count+1;
            times_vet(count)=times(i);
        end
    end
        genes_vect{1}='Gene Names';
    for i=2:n+1
        genes_vect{i}=sprintf('G%d',i-1);
    end
     DEflag=zeros(n,1);
     DEflag(r,1)=1;
   
    
    if ispc
    letters={'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z',...
        'AA','AB','AC','AD','AE','AF','AG','AH','AI','AJ','AK','AL','AM','AN','AO','AP','AQ','AR','AS','AT','AU','AV','AW','AX','AY','AZ',...
        'BA','BB','BC','BD','BE','BF','BG','BH','BI','BJ','BK','BL','BM','BN','BO','BP','BQ','BR','BS','BT','BU','BV','BW','BX','BY','BZ',...
        'CA','CB','CC','CD','CE','CF','CG','CH','CI','CJ','CK','CL','CM','CN','CO','CP','CQ','CR','CS','CT','CU','CV','CW','CX','CY','CZ',...
        'DA','DB','DC','DD','DE','DF','DG','DH','DI','DJ','DK','DL','DM','DN','DO','DP','DQ','DR','DS','DT','DU','DV','DW','DX','DY','DZ'};
    rangeT=sprintf('B1:%s1',letters{ntimes+1});
    xlswrite(file_matrixact,times_vet,rangeT);
    rangeG=sprintf('A1:A%d',n+1);
%     genes_vect{1}='Gene Names';
%     for i=2:n+1
%         genes_vect{i}=sprintf('G%d',i-1);
%     end
    xlswrite(file_matrixact,genes_vect',rangeG);
    rangeD=sprintf('B2:%s%d',letters{ntimes+1},n+1);
    xlswrite(file_matrixact,datamatrix,rangeD);
%      DEflag=zeros(n,1);
%      DEflag(r,1)=1;
     rangeDE=sprintf('B2:B%d',n+1);
     xlswrite(file_matrixact,genes_vect',2,rangeG);
     xlswrite(file_matrixact,{'DE Flag'},2,'B1');
     xlswrite(file_matrixact,DEflag,2,rangeDE);
  fprintf('dataset %s created!\n',file_matrixact);    
  %%%%
  %%%%salvare tutti i parametri che servono a mainreal in file_matrixact!
  %%%%
    else
% file_matrixact1='Dati_simul_D.txt';
% file_matrixact2='Dati_simul_T.txt';

       fidtxtD = fopen(file_matrixact1,'wt');
       fprintf(fidtxtD,'Gene_names/times\t');
       for itime=1:length(times_vet)
            fprintf(fidtxtD,'%f\t',times_vet(itime));
       end
        fprintf(fidtxtD,'\n');
           
        for ig=1:n
          fprintf(fidtxtD,'%s\t',genes_vect{ig+1});
         for ie=1:m
           if ~isempty(datamatrix(ig,ie))
            fprintf(fidtxtD,'%f\t',datamatrix(ig,ie));
           else
            fprintf(fidtxtD,'NaN\t');
           end    
         end
        fprintf(fidtxtD,'\n');
        end
       fclose(fidtxtD);
        fidtxtT = fopen(file_matrixact2,'wt');
        fprintf(fidtxtT,'Gene_names\t Flag\n');
        
       for ig=1:n
          fprintf(fidtxtD,'%s\t %i\n',genes_vect{ig+1},DEflag(ig));
      end
        
       fclose(fidtxtT);
    end


end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make_Analysis
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
type_estdegree
[id_rejected,BF,esttau2,Lhat,estpizerodum,estsigmadum]=Make_Analysis(datamatrix,times,repliche,Lmax,vi_global,lambda,type_fun,...
    type_method,type_sigma,type_pizero,type_estdegree,type_test,type_mod,est_par_ig,...
    gammino,soglia,x0,x1,estsigmaC,estpizeroC);
   if isempty(estpizeroC)
       estpizero=estpizerodum;
   else
        estpizero=estpizeroC;
   end
   if isempty(estsigmaC)
       estsigma=estsigmadum;
   else
       estsigma=estsigmaC;
   end
       
if max(esttau2)>x1-10
    flagtau2(index_fun,index_run)=1;
end

    if ~isempty(id_rejected)
     [aa,ii]=sort(BF(id_rejected));
     sort_id_rejected=id_rejected(ii);
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Store the result for each index_run
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if ~isempty(id_rejected)
     nrejected(index_run)=length(id_rejected);
     n_correctly_rejected(index_run)=length(intersect(r,id_rejected)); 
    else
     nrejected(index_run)=0;   
     n_correctly_rejected(index_run)=0;  
    end
   
 n_not_detected(index_run)=ndiff-n_correctly_rejected(index_run);
 n_falsely_rejected(index_run)=nrejected(index_run)-n_correctly_rejected(index_run);
 n_missclassified(index_run)=n_not_detected(index_run)+n_falsely_rejected(index_run);

shat=zeros(nrejected(index_run),T);

error1r=[]; % relative estimation error
error1a=[]; % absolute estimation error

error2=[];  % relative error type I
error3=[]; % relative error type II 

ind1=0;
ind2=0;
ind3=0;

if strcmp(type_fun,'legendre')
    norm_coeff=sqrt(max(times)/2);
elseif strcmp(type_fun,'fourier')
    norm_coeff=0.5*sqrt(max(times)/(2*pi));
end
%
if nrejected(index_run) >0 %if there are genes that are differentially expressed 
 [coeffs_hat,shat]=Make_Estimation(datamatrix,sort_id_rejected,BF(sort_id_rejected),Lhat(sort_id_rejected),estpizero,esttau2(sort_id_rejected),L,vi_global,type_fun,times,repliche);
   
  for i=1:nrejected(index_run) 
    coeffs_dum=coeffs_hat{i,1};
    if ismember(sort_id_rejected(i),r) 
       ind1=ind1+1;
       kkk=find(sort_id_rejected(i)==r);
       coefftrue=[];
       coefftrue=coeff{kkk};
       llest=length(coeffs_dum);
       lltrue=length(coefftrue);
         if llest>lltrue
           coefftrue=[coefftrue zeros(1,llest-lltrue)];
         elseif lltrue>llest
           coeffs_dum=[coeffs_dum zeros(1,lltrue-llest)];
         else
         end        
      error1a(ind1)=norm_coeff*sum((coeffs_dum-coefftrue).^2);  
      error1r(ind1)=sum((coeffs_dum-coefftrue).^2) / sum(coefftrue.^2);
    else  
      ind2=ind2+1;
      error2(ind2)=norm_coeff*sum(coeffs_dum.^2) ;
    end
   clear coeff_dum;  
  end
  
  id_null=setdiff(1:n,sort_id_rejected);
  inull=length(id_null);
  
  for i=1:inull
   if ismember(id_null(i),r) 
      ind3=ind3+1;
      kkk=find(id_null(i)==r);
      coefftrue=[];
      coefftrue=coeff{kkk};
      error3(ind3)=norm_coeff*sum(coefftrue.^2) ;
   end
  end
  
 if length(error1a)>0
 meantrueerror_a(index_run)=mean(error1a);
 maxtrueerror_a(index_run)=max(error1a);
 meantrueerror_r(index_run)=mean(error1r);
 maxtrueerror_r(index_run)=max(error1r);
 else
 meantrueerror_a(index_run)=0;
 maxtrueerror_a(index_run)=0;
 meantrueerror_r(index_run)=0;
 maxtrueerror_r(index_run)=0;
 end
% 
 if length(error2)>0
 meanfalselyerror(index_run)=mean(error2);
 maxfalselyerror(index_run)=max(error2);
 else
 meanfalselyerror(index_run)=0;
 maxfalselyerror(index_run)=0;
 end
% 
 if length(error3)>0
 meanfalselynegerror(index_run)=mean(error3);
 maxfalselynegerror(index_run)=max(error3);
 else
 meanfalselynegerror(index_run)=0;
 maxfalselynegerror(index_run)=0;
 end

clear error1;
clear error2;
clear aa;
clear ii;
clear id_rejected;
clear sort_id_rejected;
clear Lhat;
clear esttau;
clear BF;
clear shat
clear coeffs_hat
clear estpizero;
clear estsigma;
clear id_null;
clear inull;
end
end  %over  index_run

clear coeff;
clear truetau;
clear truedegree;
clear SNR;
clear r;
clear rdum;
clear vi_gene;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Store the result for each index_fun
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fun_nrejected(index_fun)=mean(nrejected);
fun_n_correctly_rejected(index_fun)=mean(n_correctly_rejected);
fun_n_not_detected(index_fun)=mean(n_not_detected);
fun_n_falsely_rejected(index_fun)=mean(n_falsely_rejected);
fun_n_missclassified(index_fun)=mean(n_missclassified);

fun_meantrueerror_a(index_fun)=mean(meantrueerror_a);
fun_maxtrueerror_a(index_fun)=mean(maxtrueerror_a);
fun_meantrueerror_r(index_fun)=mean(meantrueerror_r);
fun_maxtrueerror_r(index_fun)=mean(maxtrueerror_r);
fun_meanfalselyerror(index_fun)=mean(meanfalselyerror);
fun_maxfalselyerror(index_fun)=mean(maxfalselyerror);
fun_meanfalselynegerror(index_fun)=mean(meanfalselynegerror);
fun_maxfalselynegerror(index_fun)=mean(maxfalselynegerror);


waitbar(index_fun/nn_function)
end  % over index_fun

close(h)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Write simulation result in the file "file_name"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fid = fopen(file_name,'w');
 
fprintf(fid,'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n');
fprintf(fid,'....SUMMURY SET-UP SIMULATION...\n');
fprintf(fid,'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n');
fprintf(fid,' Times at which observations are available [%2.2f ',times(1));
for itim=2:length(times)-1
    fprintf(fid,'%2.2f ',times(itim));
end
fprintf(fid,'%2.2f] \n',times(end));
 
fprintf(fid,' Number of arrays available per time point [%i ',repliche(1));
for itim=2:length(times)-1
    fprintf(fid,'%i ',repliche(itim));
end
fprintf(fid,'%i] \n',repliche(end));
fprintf(fid,'----------------------------------------------------------\n');
fprintf(fid,' Basis : %s \n',type_fun);
fprintf(fid,' True maximum degree of polynomial %i :\n',L);
fprintf(fid,' Maximum degree of polynomial used in the analysis %i :\n',Lmax);

fprintf(fid,' S(t) expected to be vi-times differentiable: %i \n',vi_global);
fprintf(fid,' Poisson parameter for the prior probability g(L_i): %i  \n ',lambda);

fprintf(fid,' Type of noise added to the simulated data : %s    \n',type_error );
 
  if strcmp(type_error,'student')
   fprintf(fid, 'Degree of freedom of the scaled student %i,\n',df);
  end
fprintf(fid,' \n' );
 
fprintf(fid,' ESTIMATION METHOD = %i    \n',type_method );
 
if type_method ==2
 fprintf(fid,' Option on IG-hyper-parameters estimations %i \n',est_par_ig);
 if est_par_ig==1
 fprintf(fid,' User chosen parameter "gamma"  %i \n',gammino);
 end     
end
fprintf(fid,' Interval where searching tau2 is [ %1.5f , %1.5f ] \n',x0,x1); 
if sum(sum(flagtau2))>1
  fprintf(fid,'xxxxxxxxxxxxxxxxxxxxxxxxxx\n');
  fprintf(' tau2 estimation have been failed in %i cases over %i \n',sum(sum(flagtau2)),nn_function*nn_run);
  fprintf(' We may suggest to repeat the analysis increasing the value of x1 \n');
  fprintf(fid,' tau2 estimation have been failed in %i cases over %i \n',sum(sum(flagtau2)),nn_function*nn_run);
  fprintf(fid,' We may suggest to repeat the analysis increasing the value of x1 \n');
  fprintf(fid,'xxxxxxxxxxxxxxxxxxxxxxxxxx\n');
end
if isempty(type_sigma)
    fprintf(fid,' How to estimate sigma: custom noise estimation = %3.5f \n',estsigmaC);
else
    fprintf(fid,' How to estimate sigma= %s \n',type_sigma);
end
if isempty(type_pizero)
    fprintf(fid,' How to estimate pizero: custom proportion =%3.5f \n', estpizeroC);
else
    fprintf(fid,' How to estimate pizero=%s \n', type_pizero);
    
end
fprintf(fid,' How to estimate Lhat for each function= %s \n',type_estdegree);
fprintf(fid,' How to control multiplicity= %s \n',type_test);
fprintf(fid,' \n' );
fprintf(fid,' Number of genes we want to study  %i \n',n);
fprintf(fid,' Number of differentially expressed genes %i \n',ndiff);
fprintf(fid,' True sigma  %1.5f \n',truesigma);
fprintf(fid,' SNR in  [%1.1f, %1.1f] \n',a,b);
fprintf(fid,'                 \n');
fprintf(fid,' NUMBER OF RUNS FOR EACH DATASET   %i \n',nn_run);
fprintf(fid,' NUMBER OF DATASETS  %i \n',nn_function);
fprintf(fid,'                 \n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(fid,'---------------------------- \n');
fprintf(fid,' Mean number of rejected %10.5f ( std %10.5f) \n',mean(fun_nrejected),std(fun_nrejected));
fprintf(fid,' Mean number of correctly rejected %10.5f (std %10.5f) \n',mean(fun_n_correctly_rejected),std(fun_n_correctly_rejected));
fprintf(fid,' Mean number of not detected %10.5f ( std %10.5f)\n',mean(fun_n_not_detected),std(fun_n_not_detected));
fprintf(fid,' Mean number of falsely rejected %10.5f  (std %10.5f)\n',mean(fun_n_falsely_rejected),std(fun_n_falsely_rejected));
fprintf(fid,' Mean number of misclassified %10.5f ( std %10.5f)\n',mean(fun_n_missclassified),std(fun_n_missclassified));
fprintf(fid,'---------------------------- \n');
fprintf(fid,'                 \n');
fprintf(fid,'---------------------------- \n');
fprintf(fid,' FDR (mean %10.5f and std %10.5f)\n',mean(fun_n_falsely_rejected./fun_nrejected),std(fun_n_falsely_rejected./fun_nrejected));
fprintf(fid,' FNR (mean %10.5f and std %10.5f)\n',mean(fun_n_not_detected./(n-fun_nrejected)),std(fun_n_not_detected./(n-fun_nrejected)));
fprintf(fid,'---------------------------- \n');
fprintf(fid,'                 \n');
fprintf(fid,'---------------------------- \n');
fprintf(fid,' Average mean L2-error (absolute) estimation error (case A)   %10.5f (std %10.5f)   \n',mean(fun_meantrueerror_a),std(fun_meantrueerror_a));
%fprintf(fid,' Average max L2 (absolute) estimation error (case A)   %10.5f (std %10.5f)    \n',mean(fun_maxtrueerror_a),std(fun_maxtrueerror_a));
fprintf(fid,' Average mean L2-error (relative) estimation error (case A)   %10.5f (std %10.5f)   \n',mean(fun_meantrueerror_r),std(fun_meantrueerror_r));
%fprintf(fid,' Average max L2 (relative) estimation error (case A)   %10.5f (std %10.5f)    \n',mean(fun_maxtrueerror_r),std(fun_maxtrueerror_r));
fprintf(fid,' Average mean L2-error for false positive (case D) %10.5f (std %10.5f)\n',mean(fun_meanfalselyerror),std(fun_meanfalselyerror));
%fprintf(fid,' Average max  L2-error for false positive (case D) %10.5f (std %10.5f) \n',mean(fun_maxfalselyerror),std(fun_maxfalselyerror));
fprintf(fid,' Average mean L2-error for false negative (case B) %10.5f (std %10.5f)\n',mean(fun_meanfalselynegerror),std(fun_meanfalselynegerror));
%fprintf(fid,' Average max  L2-error for false negative (case B) %10.5f (std %10.5f) \n',mean(fun_maxfalselynegerror),std(fun_maxfalselynegerror));
fprintf(fid,'------------END OF SIMULATION---------------- \n');
fclose(fid);
fprintf('\n Symulation Performed! \n');
clear all
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
