function [lista_common,lista_union] = confronta(lista_exp,file_output_common,...
    file_output_union,overwrite)

% This utility Routine compares the output Project_GL.xls files from
% different analyses (obatined using different models or different setup parameters on the same dataset) 
% and provides as output the gene_names list common to all analyses and the
% corresponding rank order in each analysis and the list of the genes
% selected in at least one project

% Input parameters
% lista_exp: list of strings containing the files names of the Project_GL.xls we want to compare
% file_output: output xls file where store the results of the comparisons
%
% Output parameters
% lista_common:  list of gene names common to all Projects_GL.xls. Note the list is alphabetically ordered 
% lista_union:  list of gene names selected by at least one Projects_GL.xls. Note the
% list is alphabetically ordered 
% tabposition: Matrix (number of common genes) x (number of experiments to
%              be compared) containing the position/rank of each common gene in the given project

% Usage:
% lista={'Project1_GL.xls','Project2_GL.xls','Project3_GL.xls'}
% confronta(lista);

if nargin<2
    file_output_common='output_intersection.xls';
    file_output_union='output_union.xls';
end



nexp=length(lista_exp);


%fprintf('%i \n',nexp)
for i=1:nexp
fprintf('Including experiment %s \n',lista_exp{i})
end

if ispc
[num1,txt1]=xlsread(lista_exp{1});
[num2,txt2]=xlsread(lista_exp{2});
L1=txt1(2:end,2);
L2=txt2(2:end,2);
clear num1 num2 txt1 txt2
else
A=importdata(lista_exp{1});
B=importdata(lista_exp{2});
L1 = A.textdata(2:end,2);
L2 = B.textdata(2:end,2);
clear A B
end



lista_common=intersect(L1,L2);
lista_union=union(L1,L2);

if nexp>2 
 for i=3:nexp
  if ispc
  [num,txt]=xlsread(lista_exp{i});
  L=txt(2:end,2);
  else
   C=importdata(lista_exp{i});
   L=C.textdata(2:end,2);  
  end
  lista_common=intersect(lista_common,L);
  lista_union=union(lista_union,L);
  clear num txt C L
 end
end

mytab=zeros(length(lista_common),nexp);
fprintf('Number of experiments= %i \n', nexp);
fprintf('Number of genes common to all analises = %i \n', length(lista_common));
fprintf('Number of genes selected in at least one analysis = %i \n',length(lista_union));

for i=1:nexp
    if ispc
     [num,txt]=xlsread(lista_exp{i});
     [genescommon,I1,I2]=intersect(txt(2:end,2),lista_common);
    
    else
        A=importdata(lista_exp{i});
        nomi=A.textdata(2:end,2);
        [genescommon,I1,I2]=intersect(nomi,lista_common);
    end
     mytab(:,i)=I1;
     clear num txt genescommon I1 I2 A nomi
end

dum={'Gene_name'};
if overwrite==1
    idc=fopen(file_output_common);
    if idc~=-1
        fclose(idc);
        delete(file_output_common)
    end
    idu=fopen(file_output_union);
    if idu~=-1
        fclose(idu);
        delete(file_output_union)
    end
end

if ispc
xlswrite(file_output_common,dum,1,'A1')
xlswrite(file_output_common,lista_exp',1,'B1')

RangeG=sprintf('A2:A%d',length(lista_common)+1);
xlswrite(file_output_common,lista_common,1,RangeG);
xlswrite(file_output_common,mytab,1,'B2');

xlswrite(file_output_union,lista_union);
else
fidtxtc = fopen(file_output_common,'wt'); 
     fprintf(fidtxtc,'Gene_Name\t');
    for ilist=1:nexp
      fprintf(fidtxtc,'%s\t',lista_exp{ilist});
    end
    fprintf(fidtxtc,'\n');
    for ig=1:length(lista_common)
       fprintf(fidtxtc,'%s\t',lista_common{ig});
          for ilist=1:nexp
           fprintf(fidtxtc,'%i\t',mytab(ig,ilist));
          end
          fprintf(fidtxtc,'\n');
    end
fclose(fidtxtc);
fidtxtu = fopen(file_output_union,'wt'); 

for ig=1:length(lista_union)
       fprintf(fidtxtu,'%s\n',lista_union{ig});
end
fclose(fidtxtu);
end

fprintf('Results saved in \n %s and \n %s \n',file_output_common,file_output_union);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2006. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as it is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
