function [Flag_out]=display_gene_profile(gene_name,raw_data_file,hat_data_file,nfig,gsym,gcol,rsym,rcol,psym,pcol)
% 
%
%  This routine makes the plot of the temporal profile for a given gene in the figure number  nfig
% display_gene_profile(gene_name,raw_data_file,hat_data_file,nfig)

%  Input:  
%     gene_name: String containing the gene to be displayed
%     raw_data_file: String with the name of the  .xls file containing the
%                    data that have been analized (the file has to be in the BATS format)
%     hat_data_file: String with the name of the  .xls file containing the
%                    Results of the estimated profiles after having been analyzed with BATS (it is a Project_SH.xls generated by BATS-Analysis)
%     nfig: integer number for figure  
%  Output: 
%      is the figure number nfig 
 
% Reference: 
% C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

if nargin<5
    gsym='.';
end
if nargin<6
    gcol='b';
end
if nargin<7
    rsym='-';
end
if nargin<8
    rcol='k';
end
if nargin<9
    psym='-';
end
if nargin<10
    pcol='r';
end

Flag_out=1;


gsymcol=union(gsym,gcol);
if strcmp(raw_data_file(end-3:end),'.xls')
    [A_raw,B_raw]=xlsread(raw_data_file);
    geni_raw=B_raw(2:end);
    times=A_raw(1,:);
    dati_raw=A_raw(2:end,:);
    clear A_raw;

elseif strcmp(raw_data_file(end-3:end),'.txt')
    A=importdata(raw_data_file);
    geni_raw=A.textdata(2:end,1);
    times=A.data(1,:);
    dati_raw=A.data(2:end,:);

end
if exist(hat_data_file,'file')==2
    if strcmp(hat_data_file(end-3:end),'.xls')
    [A_hat,B_hat]=xlsread(hat_data_file);
    geni_hat=B_hat(2:end,2);
    times_hat=A_hat(1,3:end);
    dati_hat=A_hat(2:end,3:end);
     rank_list=A_hat(2:end,1);
    elseif strcmp(hat_data_file(end-3:end),'.txt')
    A=importdata(hat_data_file);
    times_hat=A.data(1,:);
    dati_hat=A.data(2:end,:);
    geni_hat=A.textdata(2:end,2);
    rank_list=A.textdata(2:end,1);
    end
    clear B_hat
end

clear B_raw; 

for i=1:length(geni_raw)
genedum=geni_raw{i};
ispace=find(genedum==' ');
genedum(ispace)=[];
geni_raw{i}=genedum;
end
ispace=find(gene_name==' ');
gene_name(ispace)=[];


[gene, id_raw]=intersect(geni_raw,gene_name);

% times=A_raw(1,:);
% dati_raw=A_raw(2:end,:);
% clear A_raw;

if ~isempty(id_raw)
 Ratio=dati_raw(id_raw,:);
 clear dati_raw
 Flag_raw=1;
 if length(find(isnan(Ratio)))>0
    Flag_out=-1;
    fprintf('Gene %s data contains all zero values, please check your raw data file!\n',gene_name);
    return 
 end
else
 fprintf('Gene %s is not present in file %s \n',gene_name,raw_data_file);
 fprintf('Please check the gene name and/or the raw data file \n');
 Flag_out=0;
 Flag_raw=0;
 return
end
if exist(hat_data_file,'file')==2
    for i=1:length(geni_hat)
        genedum=geni_hat{i};
        ispace=find(genedum==' ');
        genedum(ispace)=[];
        geni_hat{i}=genedum;
    end
end
ispace=find(gene_name==' ');
gene_name(ispace)=[];
if exist(hat_data_file,'file')==2
    [gene, id_hat]=intersect(geni_hat,gene_name);
    clear A_raw;
    if ~isempty(id_hat)
        Shat=dati_hat(id_hat,:);
        rank=rank_list(id_hat);
        clear dati_hat
        Flag_hat=1;
    else
        fprintf('Gene %s is not present in file %s of significant genes \n',gene_name,raw_data_file);
        Flag_hat=0;
    end
else
     Flag_hat=0;
end
if ~Flag_raw && ~Flag_hat
    fprintf('Gene %s is not in the raw data file, neither in the significant list -- Please check gene name \n',gene_name)
    h=-1;
    Flag_out=0;
else
    if nargin<4||nfig<0
        nfig=figure;
        %figure(nfig)
    end
    
    h=figure(nfig);
    if Flag_raw
        plot(times,Ratio,gsymcol) % Noisy observations in blue dots 
    end
    hold on
    if Flag_hat
        
        plot(times_hat,Shat,union(psym,pcol)) % Estimated profile is in red 
    end
    plot(times,zeros(size(times)),union(rsym,rcol)) %zero reference line in black
    if(length(find(Ratio==0))==length(Ratio))
        axis([times(1) times(end) 0 length(Ratio)])
    else
        axis([times(1) times(end) min(0,min(Ratio)) max(0,max(Ratio))])
    end
    xlabel('times')
    ylabel('Log_2 Ratio')
    mytitle=[gene_name];
    if exist(hat_data_file,'file')==2
        
        if Flag_raw && Flag_hat
            legend('Raw observations','Estimated profile','Zero-reference')
            if iscell(rank)
                rank=str2double(rank{:});
            end
            R=sprintf('%i',rank);
            mytitle=[mytitle ' detected as differentially espressed in rank ',R];
        elseif Flag_raw && ~Flag_hat
            legend('Raw observations','Zero-reference')
            fprintf('Gene %s is not differentially expressed ---> there is no estimated profile \n',gene_name)
            mytitle=[mytitle ' (not differentially expressed)'];
        elseif ~Flag_raw && Flag_hat
            legend('Estimated profile','Zero-reference')
            fprintf('Gene %s is not in the raw data file -- Please check  \n',gene_name)
        end
    else
        if Flag_raw
            legend('Raw observations','Zero-reference')
        elseif ~Flag_raw
            legend('Estimated profile','Zero-reference')
            fprintf('Gene %s is not in the raw data file -- Please check  \n',gene_name)
        end
    end
    title(mytitle)
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
