function coeff=estcoeff(id,PHI,Q,q,BF,esttau2,Lhat,estpizero)

%
%  coeff=estcoeff(id,PHI,Q,q,BF,esttau2,Lhat,estpizero)
%
%  This routine estimates the coefficients of the polynomial representing
%  the id-th gene profile in the selected basis function 
%
%  Input:
%     id        :index of the gene of which coefficients have to be estimated
%     PHI       :2D array output of subroutine make_matrix_PHI in the selected basis function 
%     Q         :2D array output of subroutine make_matrix_Q in the  selected basis function 
%     q         :1D vector output of the subroutine make_vector_q
%     BF        :1D array of Bayes factor of dimension number_gene x 1
%     esttau2   :1D vector of the estimated tau^2_i
%     Lhat      :1D array of estimated degree of dimension  number_genes x 1
%     estpizero :estimated proportion of not differentially expressed genes
 
%  Output: 
%     coeff     :estimated coefficients of the polynomial representing the id-th gene   
%    
%  
% Reference: C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

H=PHI(1:Lhat+1,1:Lhat+1)+Q(1:Lhat+1,1:Lhat+1).*(1/esttau2);
H=inv(H);
rat=(1-estpizero)./estpizero;
coeff=q(id,1:Lhat+1)*H* (rat/(rat+BF));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


