 function esttau2=estimationtau_fast(q,PHI,Q,Zzero,sigma,vi,lambda,caseoption,param,x0,x1)
%
%  esttau2=estimationtau_fast(q,PHI,Q,Zzero,sigma,vi,lambda,caseoption,param,x0,x1)
%
%  This routine estimates tau^2_i for i=1,...,number_genes maximizing the  marginal pdf of the data
%  This maximization is done for each gene separately
%
%  Input:
%      q          :1D array output of subroutine make_vector_q
%      PHI        :2D array output of subroutine make_matrix_PHI
%      Q          :2D array output of subroutine make_matrix_Q
%      Zzero      :1D vector (of the same size of q) which contain the l_2-norm of each row of the datamatrix
%      sigma      :standard deviation of the noise
%      vi         :expected degree of differentiability of the function profile
%      lambda     :Poisson parameter for the prior probability on the polynomial degree
%      caseoption :Type of method
%                   1 --> delta distribution on sigma^2;
%                   2 --> IG distribution on sigma^2;
%                   3 --> Exponential dist on sigma^2
%      param      :specific prior parameter
%                   if caseoption==1  param=[]
%                   if  caseoption==2  param=[b, gammino]
%                   if  caseoption==3  param=[mu]
%      x0,x1       : are extremes of the interval where the minimization of the marginal pdf is done to estimate tau2
%    Output: 
%      esttau2      :1D array of dimension  number_genes x 1 containing  tau_i^2
%     
% Reference: C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

 if nargin<10
     x0=0.0001;
     x1=100;
 end
 
 degree=size(PHI,1)-1;
 fact=factorial(0:degree+1); 
 n=size(q,1);
 esttau2=zeros(n,1);
 
 for i=1:n
  qdum=q(i,:);
  esttau2(i)=fminbnd(@(x) mymarginalpdf(x,PHI,Q,qdum,Zzero(i),lambda,sigma,vi,caseoption,param,fact),x0,x1);
%  clear qdum
 end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
