function [file_name_output]=filterdatanan(file_name_input,threshold,file_name_prev)

% This utility Routine filters out from the input file all genes with a
% number of missing obresvation larger than the specified threshold. 

%input parameters 
% file_name_input: string denoting the .xls file containing the results of
%                  the experiments in the BATS specified format
% threshold: positive integer, maximum number of missing values accepted for each gene

%output
%file_name_output is a .xls file (the name is obtained adding the string 'filter' to the input_file_name) where the genes containing more than threshold NaNs are removed 
%                  the output file is already in the BATS input format
if nargin<3
    if strcmp(file_name_input(end-3:end),'.xls')
        id=strfind(file_name_input,'.xls');
        id=id(1);
        file_name=file_name_input;
        file_name(id:end)=[];
    elseif strcmp(file_name_input(end-3:end),'.txt')
        id=strfind(file_name_input,'.txt');
        id=id(1);
        file_name=file_name_input;
        file_name(id:end)=[];
    else
        file_name=file_name_input;
    end
    filenameold=file_name;
    file_name=sprintf('%s_n%d',filenameold,threshold);
    if ispc && strcmp(file_name_input(end-3:end),'.xls')==1
        fprintf('Filtered data are saved in .xls format \n');
        file_name_output=[file_name '_filter.xls'];
    else
        fprintf('Filtered data are saved in .txt format \n');
        file_name_output=[file_name '_filter.txt'];
    end
else
    if strcmp(file_name_prev(end-3:end),'.xls')
        id=strfind(file_name_prev,'.xls');
        id=id(1);
        file_name=file_name_prev;
        file_name(id:end)=[];
    elseif strcmp(file_name_prev(end-3:end),'.txt')
        id=strfind(file_name_prev,'.txt');
        id=id(1);
        file_name=file_name_prev;
        file_name(id:end)=[];
    else
        file_name=file_name_prev;
    end
    filenameold=file_name;
    file_name=sprintf('%s_n%d',filenameold,threshold);
    if ispc && strcmp(file_name_input(end-3:end),'.xls')==1
        fprintf('Filtered data are saved in .xls format \n');
        file_name_output=[file_name '_filter.xls'];
    else
        fprintf('Filtered data are saved in .txt format \n');
        file_name_output=[file_name '_filter.txt'];
    end

end
file_name_output
if exist(file_name_output,'file')==2
    ButtonName=questdlg('The Filtered data file name already exists!!', ...
                       'Warning Message', ...
                       'Overwrite','Choose Again','');
     switch ButtonName,
         case 'Choose Again', 
             curpath=eval('pwd');
             startdir=fullfile(curpath,'\Examples');
[file_output,pathname]=uiputfile('*.*','Choose a Filtered Data File Name',startdir);
file_name_output=fullfile(pathname,file_output);
[file_name_output]=filterdatanan(file_name_input,threshold,file_name_output);
         case 'Overwrite',
                    
        
     end % switch
else
    if strcmp(file_name_input(end-3:end),'.xls')
        [dati,testo]=xlsread(file_name_input);
    elseif strcmp(file_name_input(end-3:end),'.txt')
        A=importdata(file_name_input);
        dati=A.data;
        testo=A.textdata;
    end
    [n,m]=size(dati);
    matnan=zeros(n,m);
    for i=1:m
        matnan(:,i)=isnan(dati(:,i));
    end
    numnan=sum(matnan,2);
    pos=find(numnan>threshold);
    dati(pos,:)=[];
    testo(pos,:)=[];
    ng=length(testo);
    fprintf('File name input %s \n',file_name_input);
    fprintf('Total number of genes in the original file %i in %i experiments \n',n-1,m);
    fprintf('Number of genes with (strictly) more than %i NaN values %i \n',threshold,length(pos));
    fprintf('Total number of genes after filtering %i \n',ng-1)
    ;fprintf('Results saved in  %s \n',file_name_output);
    if ispc && strcmp(file_name_input(end-3:end),'.xls')==1
        RangeG=sprintf('A1:A%d',ng);
        xlswrite(file_name_output,testo,1,RangeG);
        xlswrite(file_name_output,dati,1,'B1')
    else
        fidtxt = fopen(file_name_output,'wt');
        for ig=1:ng
            fprintf(fidtxt,'%s\t',testo{ig});
            for ie=1:m
                if ~isempty(dati(ig,ie))
                    fprintf(fidtxt,'%f\t',dati(ig,ie));
                else
                    fprintf(fidtxt,'NaN\t');
                end
            end
            fprintf(fidtxt,'\n');
        end
        fclose(fidtxt);
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2006. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
