function [PHI]=make_matrix_PHI(maxdegree,times,repliche,type_fun,Trange)

%
%  [PHI]=make_matrix_PHI(maxdegree,times,repliche,type_fun,Trange)

%  This routine computes the matrix PHI. PHI is a matrix of dimension (maxdegree+1)x(maxdegree+1)
%  PHI=D^t *D  where D is the dedign matrix of dimension M x (maxdegree+1)
%  where M is the number of time instants, replications being accounted.
%  
%
%  Input:  
%      maxdegree :a scalar representing the maximum degree of the  polynomial
%      times     :1D vector with the time knots
%      repliche  :1D vector (of the same size of times) with the replicates for each time instant
%      type_fun  :basis function
%                 'Legendre'
%                 'Fourier' 
%      Trange    :extremes of the Time interval
%    Output: 
%      PHI       :sqaure matrix of dimension (maxdegree+1) x (maxdegree+1)
%     

% Reference: C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

if nargin<5
    Trange(1)=0;
    Trange(2)=max(times);
end
    
if nargin<4
    type_fun='legendre';
end

if strcmp(type_fun,'legendre')
ntobs=sum(repliche);
p=zeros(maxdegree,ntobs);

xtimes=2*(times)/(Trange(2)-Trange(1))-1; %%% Rescaling of the time interval to [-1 1]  

T=length(times); 
mtimes=[];%
for k=1:T
    for l=1:repliche(k)
        mtimes=[mtimes xtimes(k)];
    end
end


for degree=0:maxdegree
 dummy=legendre(degree,mtimes)*sqrt((2*degree+1)/2); %%%In order to normalize in L2. The decay we have is for orthonormal polynomial;
 p(degree+1,:)=dummy(1,:);
end

PHI=p * p';

elseif strcmp(type_fun,'fourier')
    ftimes=[-fliplr(times) times];
    repliche=[fliplr(repliche) repliche];
    ntobs=sum(repliche);
    p=zeros(maxdegree,ntobs);
     
    xtimes=pi*(ftimes)/Trange(2);  

    T=length(xtimes); 
    mtimes=[];%
   for k=1:T
    for l=1:repliche(k)
        mtimes=[mtimes xtimes(k)];
    end
   end
    
    for degree=0:maxdegree
     if degree==0
      dummy=ones(size(mtimes))./sqrt(2*pi);  
     else
      dummy=cos(degree*mtimes)./sqrt(pi); 
     end
     p(degree+1,:)=dummy;
    end
   PHI=p*p';
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
