function [Q]=make_matrix_Q(maxdegree,vi,type_fun)

%
%  [Q]=make_matrix_Q(maxdegree,vi,type_fun)
%
%  This routine computes the matrix Q. Q is a diagonal matrix which contains
%  the decay of the coefficients of a polynomial in the selected  basis
%  when the function is vi_time differentiable
%
%  Input:  
%      maxdegree :a scalar representing the maximum degree of the  polynomial
%      vi        :a scalar representing the differentiability of the function  
%      type_fun  :basis function
%                 'legendre'
%                 'fourier'    
%    Output: 
%      Q          :diagonal matrix of dimension (maxdegree+1) x (maxdegree+1)
%     
% Reference: C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

if nargin<3
    type_fun='legendre';
end

if strcmp(type_fun,'legendre')
  v=(1:1:maxdegree+1);
  Q=diag((v).^(2*vi));
elseif strcmp(type_fun,'fourier')
  v=(1:1:maxdegree+1);
  Q=diag((v).^(2*vi));    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
