function [R,Rejected]=make_step_MAP_test(B,thresh,type)

% Perform the Stepwise (Step-Up and Step-Down) MAP procedure described in F. Abramovich & C. Angelini (2005).
% i.e., find k that maximize eq. (6) in the normal data case.

% input data
% B            n vector of Bayes Factor 
% Thresh       adaptive threshold for the stepwise procedure See eq. (7)
% type         string denoting the type of stepwise procedure (it can assume the values 'Step-Up' or 'Step-Down')
% 
% Output data
% R            Number of significant hypotheses
% Rejected     Indexes of the hypotheses to de rejected

% Reference: F. Abramovich & C. Angelini, (2005)  Bayesian MAP multiple testing procedure


n=length(B);

[B_sorted, index]=sort(B);

if strcmp(type,'Step-Up')
kk=[];
kk=find(B_sorted < thresh);
if ~isempty(kk)
    R=kk(length(kk));
else
    R=0;
end
elseif strcmp(type,'Step-Down')
  kk=[];
  kk=find(B_sorted > thresh);
  if ~isempty(kk)
   R=kk(length(1))-1;
  else
    R=n;
  end
end    

n_positive=R;
if R>0
   Rejected=index(1:R);
else
   Rejected=[];
end

% Copyright F. Abramovich & C. Angelini (2005)