function fun=mymargpdf_small_zzero(x,m,Zzero,iii)
%
%  fun=mymargpdf_small_zzero(x,m,Zzero,iii)
%
%  This routine evaluates  a function proportional to the  pdf of
%  the data for each gene taking into account only the gene profile whose
%  l_2 norm is small than the "soglia" selected in the main routine.
%  This marginal pdf will be maximized in order to estimate b and gammino.
%
%  Input:
%     x     :x=[gammino b] 
%     m     :total number of time instant, replication being accounted. m=sum(repliche)  
%     Zzero :l_2 norm of each row of the datamatrix
%     iii   :indeces of the row whose l_2 norm is small then soglia
%    Output: 
%     fun   :fun(1)= estimated gammino and fun(2)= estimated b

% Reference: 
% C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

Nstar=length(iii);
Mstar=Nstar*m;

gammino=x(1);
b=x(2);

dum1=Nstar*(log(gamma(m/2+gammino))-log(gamma(gammino)))-(Mstar/2)*log(b);
dum2=(gammino+m/2)*sum(log(1+Zzero(iii)/(2*b)));

fun=-dum1+dum2;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
