function [h]=plot_profile(id,Ratio,tm,array,nfig)
%
%  plot_profile(id,Ratio,tm,array,nfig)
%
%  This routine makes the plot of the id-th profile in the figure number  nfig
%
%  Input:  
%      id     :index of the gene (i.e. number of row) we have to plot
%      Ratio  :2D data array
%      tm     :1D array with the time knots
%      array  :1D array of the same size of tm of the replication of each time knots
%      nfig   :number of the figure where the profile have to be plotted 
%  Output: 
%      is the figure number nfig 
 
% Reference: 
% C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

if nargin<5
 nfig=figure;
 figure(nfig)
end
h=figure(nfig);
m=length(tm);
index=0;
meanva=zeros(m,1);
for k=1:m
      for j=index+1:index+array(k)
         plot(tm(k),Ratio(id,j),'.b') % noisy observations in blue dots 
         hold on
      end
       meanva(k)=mean(Ratio(id,index+1:index+array(k)));
index=index+array(k);
end
hold on

plot(tm,meanva,'r') % average values for the observation in red
plot(tm,zeros(m,1),'k') %zero reference line in black

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
