function  [sigma,s]=sigmaest(matrix,type)
%
%  [sigma,s]=sigmaest2(matrix,type,repliche)
%
%  This routine estimates the standard deviation of the noise
%
%  Input:  
%      matrix     :2D data array
%      type       :option on how to estimate sigma.
%                  'standard'--> standard deviation of the noise
%                  'mad' --> median absolute deviation of the noise.
%    Output: 
%      s          :1D vector containing the estimate of sigma for each column (i.e. for each microarray)
%      sigma      :estimated value of global sigma
 
% Reference: 
% C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

[n m]=size(matrix);
if strcmp(type,'standard')
  for j=1:m
    s(j)=std(matrix(:,j));
  end
  sigma=mean(s);
elseif strcmp(type,'mad') 
  center=repmat(median(matrix,1),n,1);
  s=1.4826*median(abs(matrix-center));
  sigma=mean(s);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
